Attribute VB_Name = "VBVSL_WinAPI"
Option Explicit

'--------------------------------------------------------
' Declarations of some Win API constants and functions
' Plus some associated helper functions
'--------------------------------------------------------

Declare Function SetParent& Lib "user32" _
                 (ByVal hWndChild As Long, ByVal hWndNewParent As Long)

Declare Function GetWindowLong& Lib "user32" Alias "GetWindowLongA" _
                 (ByVal HWnd As Long, ByVal nIndex As Long)

Declare Function SetWindowLong& Lib "user32" Alias "SetWindowLongA" _
                (ByVal HWnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long)
                

Declare Function MoveWindow& Lib "user32" _
                 (ByVal HWnd As Long, ByVal nLeft As Long, ByVal nTop As Long, _
                  ByVal nWidth As Long, ByVal nHeight As Long, ByVal bRepaint As Long)
                  
Public Type RECT
  left As Long
  top As Long
  right As Long
  bottom As Long
End Type
                  
Public Declare Function GetWindowRect Lib "user32" (ByVal HWnd As Long, lpRect As RECT) As Long
                  
Public Declare Function SetWindowPos Lib "user32" ( _
                 ByVal HWnd As Long, ByVal hWndInsertAfter As Long, _
                 ByVal x As Long, ByVal y As Long, _
                 ByVal cx As Long, ByVal cy As Long, ByVal wFlags As Long) As Long
                 
Public Declare Function GetWindow Lib "user32" (ByVal HWnd As Long, ByVal wCmd As Long) As Long

Public Declare Function GetParent Lib "user32" (ByVal HWnd As Long) As Long

Public Declare Function GetClassName Lib "user32" Alias "GetClassNameA" _
                         (ByVal HWnd As Long, ByVal lpClassName As String, ByVal nMaxCount As Long) As Long


Public Enum GW_xxx
  GW_HWNDFIRST = 0
  GW_HWNDLAST = 1
  GW_HWNDNEXT = 2
  GW_HWNDPREV = 3
  GW_OWNER = 4
  GW_CHILD = 5
  GW_MAX = 5
End Enum

Public Enum GWL
  GWL_WNDPROC = -4
  GWL_HINSTANCE = -6
  GWL_HWNDPARENT = -8
  GWL_Style = -16
  GWL_ExStyle = -20
  GWL_USERDATA = -21
  GWL_ID = -12
End Enum

Public Enum StyleBits
  WS_OVERLAPPED = 0
  WS_MAXIMIZEBOX = &H10000
  WS_MINIMIZEBOX = &H20000
  WS_THICKFRAME = &H40000
  WS_SYSMENU = &H80000
  WS_HSCROLL = &H100000
  WS_VSCROLL = &H200000
  WS_DLGFRAME = &H400000
  WS_BORDER = &H800000
  WS_MAXIMIZE = &H1000000
  WS_CLIPCHILDREN = &H2000000
  WS_CLIPSIBLINGS = &H4000000
  WS_DISABLED = &H8000000
  WS_VISIBLE = &H10000000
  WS_MINIMIZE = &H20000000
  WS_CHILD = &H40000000
  WS_POPUP = &H80000000
  
  WS_CAPTION = WS_BORDER Or WS_DLGFRAME
  WS_TILED = WS_OVERLAPPED
  WS_ICONIC = WS_MINIMIZE
  WS_SIZEBOX = WS_THICKFRAME

  ' { Common Window Styles }
  WS_OVERLAPPEDWINDOW = (WS_OVERLAPPED Or WS_CAPTION Or WS_SYSMENU Or WS_THICKFRAME Or WS_MINIMIZEBOX Or WS_MAXIMIZEBOX)
  WS_TILEDWINDOW = WS_OVERLAPPEDWINDOW
  WS_POPUPWINDOW = (WS_POPUP Or WS_BORDER Or WS_SYSMENU)
  WS_CHILDWINDOW = (WS_CHILD)
End Enum

Public Enum ExStyleBits
  WS_EX_DLGMODALFRAME = 1
  WS_EX_NOPARENTNOTIFY = 4
  WS_EX_TOPMOST = &H8
  WS_EX_ACCEPTFILES = &H10
  WS_EX_TRANSPARENT = &H20
  WS_EX_MDICHILD = &H40
  WS_EX_TOOLWINDOW = &H80
  WS_EX_WINDOWEDGE = &H100
  WS_EX_CLIENTEDGE = &H200
  WS_EX_CONTEXTHELP = &H400
  WS_EX_RIGHT = &H1000
  WS_EX_LEFT = 0
  WS_EX_RTLREADING = &H2000
  WS_EX_LTRREADING = 0
  WS_EX_LEFTSCROLLBAR = &H4000
  WS_EX_RIGHTSCROLLBAR = 0
  WS_EX_CONTROLPARENT = &H10000
  WS_EX_STATICEDGE = &H20000
  WS_EX_APPWINDOW = &H40000
  WS_EX_OVERLAPPEDWINDOW = (WS_EX_WINDOWEDGE Or WS_EX_CLIENTEDGE)
  WS_EX_PALETTEWINDOW = (WS_EX_WINDOWEDGE Or WS_EX_TOOLWINDOW Or WS_EX_TOPMOST)
End Enum

Public Enum SWP_Bits
  SWP_NOSIZE = 1
  SWP_NOMOVE = 2
  SWP_NOZORDER = 4
  SWP_NOREDRAW = 8
  SWP_NOACTIVATE = &H10
  SWP_FRAMECHANGED = &H20       '{ The frame changed: send WM_NCCALCSIZE }
  SWP_SHOWWINDOW = &H40
  SWP_HIDEWINDOW = &H80
  SWP_NOCOPYBITS = &H100
  SWP_NOOWNERZORDER = &H200     '{ Don't do owner Z ordering }
  SWP_NOSENDCHANGING = &H400     '{ Don't send WM_WINDOWPOSCHANGING }
  SWP_DRAWFRAME = SWP_FRAMECHANGED
  SWP_NOREPOSITION = SWP_NOOWNERZORDER
  SWP_DEFERERASE = &H2000
  SWP_ASYNCWINDOWPOS = &H4000
End Enum

Public Enum HWND_xxx
  HWND_TOP = 0
  HWND_BOTTOM = 1
  HWND_TOPMOST = -1 ' 0xFFFFFFFF
  HWND_NOTOPMOST = -2 ' 0xFFFFFFFE
End Enum
     
Public Function StyleBitsLookup(WS As Long) As String
Dim S As String
  S = ""
  If (WS And WS_MAXIMIZEBOX) <> 0 Then S = S + "MAXIMIZEBOX" + " + "
  If (WS And WS_MINIMIZEBOX) <> 0 Then S = S + "MINIMIZEBOX" + " + "
  If (WS And WS_THICKFRAME) <> 0 Then S = S + "THICKFRAME" + " + "
  If (WS And WS_SYSMENU) <> 0 Then S = S + "SYSMENU" + " + "
  If (WS And WS_HSCROLL) <> 0 Then S = S + "HSCROLL" + " + "
  If (WS And WS_VSCROLL) <> 0 Then S = S + "VSCROLL" + " + "
  If (WS And WS_DLGFRAME) <> 0 Then S = S + "DLGFRAME" + " + "
  If (WS And WS_BORDER) <> 0 Then S = S + "BORDER" + " + "
  If (WS And WS_MAXIMIZE) <> 0 Then S = S + "MAXIMIZE" + " + "
  If (WS And WS_CLIPCHILDREN) <> 0 Then S = S + "CLIPCHILDREN" + " + "
  If (WS And WS_CLIPSIBLINGS) <> 0 Then S = S + "CLIPSIBLINGS" + " + "
  If (WS And WS_DISABLED) <> 0 Then S = S + "DISABLED" + " + "
  If (WS And WS_VISIBLE) <> 0 Then S = S + "VISIBLE" + " + "
  If (WS And WS_MINIMIZE) <> 0 Then S = S + "MINIMIZE" + " + "
  If (WS And WS_CHILD) <> 0 Then S = S + "CHILD" + " + "
  If (WS And WS_POPUP) <> 0 Then S = S + "POPUP" + " + "
  If Len(S) > 3 Then S = left$(S, Len(S) - 3)
  StyleBitsLookup = S
End Function

Public Function StyleBitsExLookup(WS As Long) As String
Dim S As String
  S = ""
  If (WS And WS_EX_DLGMODALFRAME) <> 0 Then S = S + "DLGMODALFRAME " + " + "
  If (WS And WS_EX_NOPARENTNOTIFY) <> 0 Then S = S + "NOPARENTNOTIFY" + " + "
  If (WS And WS_EX_TOPMOST) <> 0 Then S = S + "TOPMOST" + " + "
  If (WS And WS_EX_ACCEPTFILES) <> 0 Then S = S + "ACCEPTFILES" + " + "
  If (WS And WS_EX_TRANSPARENT) <> 0 Then S = S + "TRANSPARENT" + " + "
  If (WS And WS_EX_MDICHILD) <> 0 Then S = S + "MDICHILD" + " + "
  If (WS And WS_EX_TOOLWINDOW) <> 0 Then S = S + "TOOLWINDOW" + " + "
  If (WS And WS_EX_WINDOWEDGE) <> 0 Then S = S + "WINDOWEDGE" + " + "
  If (WS And WS_EX_CLIENTEDGE) <> 0 Then S = S + "CLIENTEDGE" + " + "
  If (WS And WS_EX_CONTEXTHELP) <> 0 Then S = S + "CONTEXTHELP" + " + "
  If (WS And WS_EX_RIGHT) <> 0 Then S = S + "RIGHT" + " + "
  If (WS And WS_EX_RTLREADING) <> 0 Then S = S + "RTLREADING" + " + "
  If (WS And WS_EX_LEFTSCROLLBAR) <> 0 Then S = S + "LEFTSCROLLBAR" + " + "
  If (WS And WS_EX_CONTROLPARENT) <> 0 Then S = S + "CONTROLPARENT" + " + "
  If (WS And WS_EX_STATICEDGE) <> 0 Then S = S + "STATICEDGE" + " + "
  If (WS And WS_EX_APPWINDOW) <> 0 Then S = S + "APPWINDOW" + " + "
  If Len(S) > 3 Then S = left$(S, Len(S) - 3)
  StyleBitsExLookup = S
End Function

Public Function GetClassNameVB(HWnd As Long) As String
Dim ClassName As String, CharCount As Integer
  ClassName = "12345678901234567890123456789012345678901234567890"
  CharCount = GetClassName(HWnd, ClassName, Len(ClassName))
  GetClassNameVB = left$(ClassName, CharCount)
End Function

Public Function FindChildWindowWithClass(HWnd As Long, ClassName As String) As Long
Dim ChildHW As Long, ChildClassName As String
  FindChildWindowWithClass = 0
  ChildHW = GetWindow(HWnd, GW_CHILD)
  While ChildHW <> 0
    ChildClassName = GetClassNameVB(ChildHW)
    If ChildClassName = ClassName Then
      FindChildWindowWithClass = ChildHW
      GoTo done
    End If
    ChildHW = GetWindow(ChildHW, GW_HWNDNEXT)
  Wend
done:
End Function
